from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .resource_instance_float import ResourceInstanceFloat
    from .resource_instance_integer import ResourceInstanceInteger
    from .r_instance_with_unit import RInstanceWithUnit

@dataclass
class StatisticVals(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The averageEnergyConsumption property
    average_energy_consumption: Optional[ResourceInstanceFloat] = None
    # The averageFuelConsumption property
    average_fuel_consumption: Optional[RInstanceWithUnit] = None
    # The averageSpeed property
    average_speed: Optional[ResourceInstanceInteger] = None
    # The averageSpeedAutomatic property
    average_speed_automatic: Optional[ResourceInstanceInteger] = None
    # The distanceToEmpty property
    distance_to_empty: Optional[RInstanceWithUnit] = None
    # The distanceToEmptyBattery property
    distance_to_empty_battery: Optional[ResourceInstanceInteger] = None
    # The tripMeterAutomatic property
    trip_meter_automatic: Optional[ResourceInstanceFloat] = None
    # The tripMeterManual property
    trip_meter_manual: Optional[ResourceInstanceFloat] = None
    # The tripMeter1 property
    trip_meter1: Optional[RInstanceWithUnit] = None
    # The tripMeter2 property
    trip_meter2: Optional[RInstanceWithUnit] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> StatisticVals:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: StatisticVals
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return StatisticVals()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .resource_instance_float import ResourceInstanceFloat
        from .resource_instance_integer import ResourceInstanceInteger
        from .r_instance_with_unit import RInstanceWithUnit

        from .resource_instance_float import ResourceInstanceFloat
        from .resource_instance_integer import ResourceInstanceInteger
        from .r_instance_with_unit import RInstanceWithUnit

        fields: Dict[str, Callable[[Any], None]] = {
            "averageEnergyConsumption": lambda n : setattr(self, 'average_energy_consumption', n.get_object_value(ResourceInstanceFloat)),
            "averageFuelConsumption": lambda n : setattr(self, 'average_fuel_consumption', n.get_object_value(RInstanceWithUnit)),
            "averageSpeed": lambda n : setattr(self, 'average_speed', n.get_object_value(ResourceInstanceInteger)),
            "averageSpeedAutomatic": lambda n : setattr(self, 'average_speed_automatic', n.get_object_value(ResourceInstanceInteger)),
            "distanceToEmpty": lambda n : setattr(self, 'distance_to_empty', n.get_object_value(RInstanceWithUnit)),
            "distanceToEmptyBattery": lambda n : setattr(self, 'distance_to_empty_battery', n.get_object_value(ResourceInstanceInteger)),
            "tripMeterAutomatic": lambda n : setattr(self, 'trip_meter_automatic', n.get_object_value(ResourceInstanceFloat)),
            "tripMeterManual": lambda n : setattr(self, 'trip_meter_manual', n.get_object_value(ResourceInstanceFloat)),
            "tripMeter1": lambda n : setattr(self, 'trip_meter1', n.get_object_value(RInstanceWithUnit)),
            "tripMeter2": lambda n : setattr(self, 'trip_meter2', n.get_object_value(RInstanceWithUnit)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        from .resource_instance_float import ResourceInstanceFloat
        from .resource_instance_integer import ResourceInstanceInteger
        from .r_instance_with_unit import RInstanceWithUnit

        writer.write_object_value("averageEnergyConsumption", self.average_energy_consumption)
        writer.write_object_value("averageFuelConsumption", self.average_fuel_consumption)
        writer.write_object_value("averageSpeed", self.average_speed)
        writer.write_object_value("averageSpeedAutomatic", self.average_speed_automatic)
        writer.write_object_value("distanceToEmpty", self.distance_to_empty)
        writer.write_object_value("distanceToEmptyBattery", self.distance_to_empty_battery)
        writer.write_object_value("tripMeterAutomatic", self.trip_meter_automatic)
        writer.write_object_value("tripMeterManual", self.trip_meter_manual)
        writer.write_object_value("tripMeter1", self.trip_meter1)
        writer.write_object_value("tripMeter2", self.trip_meter2)
        writer.write_additional_data_value(self.additional_data)
    




from .dynamic_access_token_provider import DynamicAccessTokenProvider
from .static_access_token_provider import StaticAccessTokenProvider
from .volvo_authentication_provider import VolvoAuthenticationProvider


# import sys

# if sys.version_info[:2] >= (3, 8):
#     # TODO: Import directly (no need for conditional) when `python_requires = >= 3.8`
#     from importlib.metadata import PackageNotFoundError, version  # pragma: no cover
# else:
#     from importlib_metadata import PackageNotFoundError, version  # pragma: no cover

# try:
#     # Change here if project is renamed and does not equal the package name
#     dist_name = __name__
#     __version__ = version(dist_name)
# except PackageNotFoundError:  # pragma: no cover
#     __version__ = "unknown"
# finally:
#     del version, PackageNotFoundError

from .authentication import *
from .vcclient.volvo_connected_client import VolvoConnectedClient


import json

def add_extra_schemas(data: any) -> any:
    # Adding RInstanceNumberWithUnit object
    # data["components"]["schemas"]["ResourceInstanceInt"] = {
    #     "type": "object",
    #     "properties": {
    #         "timestamp": {
    #             "type": "string",
    #             "format": "date-time"
    #         },
    #         "value": {
    #             "type": "number",
    #             "format": "int32"
    #         },
    #         "unit": {
    #             "type": "string"
    #         }
    #     }
    # }

    return data

def add_extra_properties_to_statistics(data: any) -> any:
    # Additional properties to the StatisticVals object
    for key in ['averageEnergyConsumption', 'tripMeterManual', 'tripMeterAutomatic']:
        data["components"]["schemas"]["StatisticVals"]["properties"][key] = {
            "$ref": "#/components/schemas/ResourceInstanceFloat"
        }
    
    # Changing type of the properties to RInstanceNumberWithUnit
    for key in ['averageSpeed', 'averageSpeedAutomatic', 'distanceToEmptyBattery']:
        data["components"]["schemas"]["StatisticVals"]["properties"][key] = {
            "$ref": "#/components/schemas/ResourceInstanceInteger"
        }

    return data

def add_extra_properties_to_fuel(data:any) -> any:
    for key in ['batteryChargeLevel']:
        data["components"]["schemas"]["FuelAmount"]["properties"][key] = {
            "$ref": "#/components/schemas/ResourceInstanceFloat"
        }
    return data


def transform_openapi_specs_objects_to_data(data: any, keys: list[str]) -> any:
   
    for key in keys:
        current_object = data["components"]["schemas"][key]
        if current_object is not None:
            new_object = {
                "type": "object",
                "properties": {
                    "data": current_object
                }
            }
            data["components"]["schemas"][key] = new_object
    
    return data

def transform_openapi_specs_objects_to_data_array(data: any, keys: list[str]) -> any:
    for key in keys:
        current_object = data["components"]["schemas"][key]
        if current_object is not None:
            new_object = {
                "type": "object",
                "properties": {
                    "data": {
                        "type": "array",
                        "items": current_object
                    }
                }
            }
            data["components"]["schemas"][key] = new_object
    
    return data

def transform_openapi_to_actual_api(file_path: str) -> None:
    with open(f'/workspaces/volvo_connected/.build/{file_path}', "r") as file:
        data = json.load(file)

    data = add_extra_schemas(data)
    data = add_extra_properties_to_statistics(data)
    data = add_extra_properties_to_fuel(data)

    data = transform_openapi_specs_objects_to_data_array(data, ["VehicleVin"])
    data = transform_openapi_specs_objects_to_data(data, [
        "Windows", 
        "Doors", 
        "EngineStatus", 
        "VehicleDetails", 
        "ExteriorWarning", 
        "OdometerValue",
        "FuelAmount", 
        "EngineDiagnostics",
        "Diagnostics", 
        "BrakeStatus",
        "CommandAccessibility",
        "Invoke",
        "InvokeUnlock"
      ])

    with open(f'/workspaces/volvo_connected/.build/volvo_connected_specs.json', "w") as file:
        json.dump(data, file, indent=2)

transform_openapi_to_actual_api("connected-vehicle-c3-specification_original.json")


from typing import Any, Dict
from kiota_abstractions.headers_collection import HeadersCollection
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.authentication.authentication_provider import AuthenticationProvider
from kiota_abstractions.authentication.base_bearer_token_authentication_provider import BaseBearerTokenAuthenticationProvider
from kiota_abstractions.authentication.access_token_provider import AccessTokenProvider

class VolvoAuthenticationProvider(BaseBearerTokenAuthenticationProvider):
    """ Volvo authentication provider that adds the VCC API key to the headers, and calls the access token provider to add the bearer token.
    
    ARGS:
        vcc_api_key (str): The VCC API key (see https://developer.volvocars.com/apis/connected-vehicle/v2/details/)
        access_token_provider (AccessTokenProvider): The access token provider to use to get the bearer token

    """

    VCC_API_KEY_HEADER = "vcc-api-key"
    AUTHORIZATION_HEADER = "Authorization"
    def __init__(self, vcc_api_key: str, access_token_provider: AccessTokenProvider):
        if not vcc_api_key:
            raise ValueError("VCC API key cannot be null or empty")
        if not access_token_provider:
            raise TypeError("Access token provider cannot be null")

        super().__init__(access_token_provider)
        #self._access_token_provider = access_token_provider
        self._vcc_api_key = vcc_api_key

    async def authenticate_request(self, request: RequestInformation, additional_authentication_context: Dict[str, Any] = {}) -> None:
        """" Authenticates the request by adding the VCC API key to the headers, and calling the access token provider to add the bearer token """

        # Check if the request is null
        if not request:
            raise Exception("Request cannot be null")
        
        # Initialize the headers collection if it does not exist
        if not request.headers:
            request.headers = HeadersCollection()

        # Remove the VCC API key if it exists
        # if self.VCC_API_KEY_HEADER in request.headers:
        #     request.headers.remove(self.VCC_API_KEY_HEADER)
        
        # Add the VCC API key to the headers
        request.headers.add('vcc-api-key', [self._vcc_api_key])

        # if not request.headers.contains(self.AUTHORIZATION_HEADER):
        #     token = await self._access_token_provider.get_authorization_token(
        #         request.url, additional_authentication_context
        #     )
        #     if token:
        #         request.headers.add(self.AUTHORIZATION_HEADER, [f'Bearer {token}'])

        # Call the base class authenticate_request method (which calls the access token provider)
        await super().authenticate_request(request, additional_authentication_context)




"""
    Dummy conftest.py for volvo_connected.

    If you don't know what this is for, just leave it empty.
    Read more about conftest.py under:
    - https://docs.pytest.org/en/stable/fixture.html
    - https://docs.pytest.org/en/stable/writing_plugins.html
"""

# import pytest
import sys
if (sys.path[1] != "/workspaces/volvo_connected/src"):
    sys.path.insert(1, "/workspaces/volvo_connected/src")


from enum import Enum

class InvokeUnlock_data_invokeStatus(str, Enum):
    WAITING = "WAITING",
    RUNNING = "RUNNING",
    COMPLETED = "COMPLETED",
    REJECTED = "REJECTED",
    UNKNOWN = "UNKNOWN",
    TIMEOUT = "TIMEOUT",
    CONNECTION_FAILURE = "CONNECTION_FAILURE",
    VEHICLE_IN_SLEEP = "VEHICLE_IN_SLEEP",
    UNLOCK_TIME_FRAME_PASSED = "UNLOCK_TIME_FRAME_PASSED",
    UNABLE_TO_LOCK_DOOR_OPEN = "UNABLE_TO_LOCK_DOOR_OPEN",
    EXPIRED = "EXPIRED",
    SENT = "SENT",
    NOT_SUPPORTED = "NOT_SUPPORTED",
    CAR_IN_SLEEP_MODE = "CAR_IN_SLEEP_MODE",
    DELIVERED = "DELIVERED",
    DELIVERY_TIMEOUT = "DELIVERY_TIMEOUT",
    SUCCESS = "SUCCESS",
    CAR_TIMEOUT = "CAR_TIMEOUT",
    CAR_ERROR = "CAR_ERROR",
    NOT_ALLOWED_PRIVACY_ENABLED = "NOT_ALLOWED_PRIVACY_ENABLED",
    NOT_ALLOWED_WRONG_USAGE_MODE = "NOT_ALLOWED_WRONG_USAGE_MODE",
    INVOCATION_SPECIFIC_ERROR = "INVOCATION_SPECIFIC_ERROR",




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from ....models.command_list_response import CommandListResponse
    from ....models.error_response import ErrorResponse
    from .climatization_start.climatization_start_request_builder import ClimatizationStartRequestBuilder
    from .climatization_stop.climatization_stop_request_builder import ClimatizationStopRequestBuilder
    from .engine_start.engine_start_request_builder import EngineStartRequestBuilder
    from .engine_stop.engine_stop_request_builder import EngineStopRequestBuilder
    from .flash.flash_request_builder import FlashRequestBuilder
    from .honk.honk_request_builder import HonkRequestBuilder
    from .honk_flash.honk_flash_request_builder import HonkFlashRequestBuilder
    from .lock.lock_request_builder import LockRequestBuilder
    from .lock_reduced_guard.lock_reduced_guard_request_builder import LockReducedGuardRequestBuilder
    from .unlock.unlock_request_builder import UnlockRequestBuilder

class CommandsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /vehicles/{vin}/commands
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new CommandsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/vehicles/{vin}/commands", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[CommandListResponse]:
        """
        Used to list the commands which can be sent to the vehicle
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[CommandListResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        from ....models.error_response import ErrorResponse

        error_mapping: Dict[str, type[ParsableFactory]] = {
            "400": ErrorResponse,
            "401": ErrorResponse,
            "403": ErrorResponse,
            "404": ErrorResponse,
            "405": ErrorResponse,
            "409": ErrorResponse,
            "415": ErrorResponse,
            "422": ErrorResponse,
            "500": ErrorResponse,
            "503": ErrorResponse,
            "504": ErrorResponse,
        }
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ....models.command_list_response import CommandListResponse

        return await self.request_adapter.send_async(request_info, CommandListResponse, error_mapping)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Used to list the commands which can be sent to the vehicle
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> CommandsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: CommandsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return CommandsRequestBuilder(self.request_adapter, raw_url)
    
    @property
    def climatization_start(self) -> ClimatizationStartRequestBuilder:
        """
        The climatizationStart property
        """
        from .climatization_start.climatization_start_request_builder import ClimatizationStartRequestBuilder

        return ClimatizationStartRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def climatization_stop(self) -> ClimatizationStopRequestBuilder:
        """
        The climatizationStop property
        """
        from .climatization_stop.climatization_stop_request_builder import ClimatizationStopRequestBuilder

        return ClimatizationStopRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def engine_start(self) -> EngineStartRequestBuilder:
        """
        The engineStart property
        """
        from .engine_start.engine_start_request_builder import EngineStartRequestBuilder

        return EngineStartRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def engine_stop(self) -> EngineStopRequestBuilder:
        """
        The engineStop property
        """
        from .engine_stop.engine_stop_request_builder import EngineStopRequestBuilder

        return EngineStopRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def flash(self) -> FlashRequestBuilder:
        """
        The flash property
        """
        from .flash.flash_request_builder import FlashRequestBuilder

        return FlashRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def honk(self) -> HonkRequestBuilder:
        """
        The honk property
        """
        from .honk.honk_request_builder import HonkRequestBuilder

        return HonkRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def honk_flash(self) -> HonkFlashRequestBuilder:
        """
        The honkFlash property
        """
        from .honk_flash.honk_flash_request_builder import HonkFlashRequestBuilder

        return HonkFlashRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def lock(self) -> LockRequestBuilder:
        """
        The lock property
        """
        from .lock.lock_request_builder import LockRequestBuilder

        return LockRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def lock_reduced_guard(self) -> LockReducedGuardRequestBuilder:
        """
        The lockReducedGuard property
        """
        from .lock_reduced_guard.lock_reduced_guard_request_builder import LockReducedGuardRequestBuilder

        return LockReducedGuardRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def unlock(self) -> UnlockRequestBuilder:
        """
        The unlock property
        """
        from .unlock.unlock_request_builder import UnlockRequestBuilder

        return UnlockRequestBuilder(self.request_adapter, self.path_parameters)
    
    @dataclass
    class CommandsRequestBuilderGetRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from typing import Any, Dict
from kiota_abstractions.authentication.access_token_provider import AccessTokenProvider, AllowedHostsValidator
class StaticAccessTokenProvider(AccessTokenProvider):
    """ Static access token provider that returns a fixed access token.	

    ARGS:	
        access_token (str): The access token to return	
    """
    def __init__(self, access_token: str):
        if not access_token:
            raise ValueError("Access token cannot be null")
        super().__init__()
        self._access_token = access_token
    
    async def get_authorization_token(
        self, uri: str, additional_authentication_context: Dict[str, Any] = {}
    ) -> str:
        return self._access_token
    
    async def get_allowed_hosts_validator(self) -> AllowedHostsValidator:
        return AllowedHostsValidator(["api.volvocars.com"])



from typing import Any, Callable, Dict
from kiota_abstractions.authentication.access_token_provider import AccessTokenProvider, AllowedHostsValidator
class DynamicAccessTokenProvider(AccessTokenProvider):
    """ Dynamic access token provider that calls a function to get the access token.	

    ARGS:	
        access_token_function (Callable[[str], str]): The function to call to get the access token. The function takes a URI as a parameter and returns the access token.
    """
    def __init__(self, access_token_function: Callable[[str], str]):
        if not access_token_function:
            raise ValueError("Access token function cannot be null")
        super().__init__()
        self._access_token_function = access_token_function
    
    async def get_authorization_token(
        self, uri: str, additional_authentication_context: Dict[str, Any] = {}
    ) -> str:
        return self._access_token_function(uri)
    
    async def get_allowed_hosts_validator(self) -> AllowedHostsValidator:
        return AllowedHostsValidator(["api.volvocars.com"])



import pytest
import asyncio
import pytest_asyncio
from kiota_bundle.default_request_adapter import DefaultRequestAdapter
from kiota_abstractions.authentication import AnonymousAuthenticationProvider
from os import getenv
import sys
if (sys.path[1] != "/workspaces/volvo_connected/src"):
    sys.path.insert(1, "/workspaces/volvo_connected/src")

from volvo_connected import (
  StaticAccessTokenProvider,
  VolvoAuthenticationProvider,
  VolvoConnectedClient
)

def get_client() -> VolvoConnectedClient:
    vcc: str = getenv("VOLVO_VCC_API_KEY");
    token: str = getenv("VOLVO_TOKEN");
    auth_provider = VolvoAuthenticationProvider(vcc, StaticAccessTokenProvider(token))
    return VolvoConnectedClient(DefaultRequestAdapter(auth_provider))

def test_no_request_adapter():
    with pytest.raises(TypeError):
        VolvoConnectedClient(None)

@pytest.mark.asyncio
async def test_get_vehicles():
    # Arrange
    client = get_client()

    # Act
    vehicles = await client.vehicles.get()

    # Assert
    assert vehicles is not None
    assert vehicles.data is not None 
    assert vehicles.data[0].vin is not None

@pytest.mark.asyncio
async def test_get_vehicle():
    # Arrange
    client = get_client()

    # Act
    vehicle = await client.vehicles.by_vin(getenv("VOLVO_VIN")).get()

    # Assert
    assert vehicle is not None
    assert vehicle.data is not None
    assert vehicle.data.vin is not None
    assert vehicle.data.images.exterior_image_url is not None
    assert vehicle.data.images.internal_image_url is not None
    assert vehicle.data.descriptions.model is not None

@pytest.mark.asyncio
async def test_get_windows():
    # Arrange
    client = get_client()

    # Act
    windows = await client.vehicles.by_vin(getenv("VOLVO_VIN")).windows.get()

    # Assert
    assert windows is not None
    assert windows.data is not None
    assert windows.data.front_left_window is not None
    assert windows.data.front_left_window.value is not None

@pytest.mark.asyncio
async def test_get_doors():
    # Arrange
    client = get_client()

    # Act
    doors = await client.vehicles.by_vin(getenv("VOLVO_VIN")).doors.get()

    # Assert
    assert doors is not None
    assert doors.data is not None
    assert doors.data.front_left_door is not None
    assert doors.data.front_left_door.value is not None

@pytest.mark.asyncio
async def test_get_engine_status():
    # Arrange
    client = get_client()

    # Act
    result = await client.vehicles.by_vin(getenv("VOLVO_VIN")).engine_status.get()

    # Assert
    assert result is not None
    assert result.data is not None
    assert result.data.engine_status is not None
    assert result.data.engine_status.value is not None

@pytest.mark.asyncio
async def test_get_engine():
    # Arrange
    client = get_client()

    # Act
    result = await client.vehicles.by_vin(getenv("VOLVO_VIN")).engine.get()

    # Assert
    assert result is not None
    assert result.data is not None
    assert result.data.engine_coolant_level_warning.value is not None
    assert result.data.oil_level_warning.value is not None

@pytest.mark.asyncio
async def test_get_warnings():
    # Arrange
    client = get_client()

    # Act
    result = await client.vehicles.by_vin(getenv("VOLVO_VIN")).warnings.get()

    # Assert
    assert result is not None
    assert result.data is not None
    assert result.data.brake_light_center_warning is not None
    assert result.data.brake_light_center_warning.value is not None

@pytest.mark.asyncio
async def test_get_tyres():
    # Arrange
    client = get_client()

    # Act
    result = await client.vehicles.by_vin(getenv("VOLVO_VIN")).tyres.get()

    # Assert
    assert result is not None
    assert result.data is not None
    assert result.data.front_left is not None
    assert result.data.front_left.value is not None

@pytest.mark.asyncio
async def test_get_statistics():
    # Arrange
    client = get_client()

    # Act
    result = await client.vehicles.by_vin(getenv("VOLVO_VIN")).statistics.get()

    # Assert
    assert result is not None
    assert result.data is not None
    assert result.data.average_speed_automatic is not None
    assert result.data.average_speed_automatic.value is not None
    assert result.data.average_speed is not None
    assert result.data.average_speed.value is not None

@pytest.mark.asyncio
async def test_get_odometer():
    # Arrange
    client = get_client()

    # Act
    result = await client.vehicles.by_vin(getenv("VOLVO_VIN")).odometer.get()

    # Assert
    assert result is not None
    assert result.data is not None
    assert result.data.odometer is not None
    assert result.data.odometer.value is not None

@pytest.mark.asyncio
async def test_get_fuel():
    # Arrange
    client = get_client()

    # Act
    result = await client.vehicles.by_vin(getenv("VOLVO_VIN")).fuel.get()

    # Assert
    assert result is not None
    assert result.data is not None
    assert result.data.battery_charge_level is not None
    assert result.data.battery_charge_level.value is not None

@pytest.mark.asyncio
async def test_get_commands():
    # Arrange
    client = get_client()

    # Act
    result = await client.vehicles.by_vin(getenv("VOLVO_VIN")).commands.get()

    # Assert
    assert result is not None
    assert result.data is not None
    assert result.data[0].command is not None
    assert result.data[0].href is not None

@pytest.mark.asyncio
async def test_get_command_accessibility():
    # Arrange
    client = get_client()

    # Act
    result = await client.vehicles.by_vin(getenv("VOLVO_VIN")).command_accessibility.get()

    # Assert
    assert result is not None
    assert result.data is not None
    assert result.data.availability_status or result.data.unavailable_reason is not None

# if you want to run these tests with just F5, uncomment this
#asyncio.run(test_get_vehicles())


from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .descriptions import Descriptions
    from .images import Images

@dataclass
class VehicleDetails_data(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The batteryCapacityKWH property
    battery_capacity_k_w_h: Optional[float] = None
    # The descriptions property
    descriptions: Optional[Descriptions] = None
    # The externalColour property
    external_colour: Optional[str] = None
    # The fuelType property
    fuel_type: Optional[str] = None
    # The gearbox property
    gearbox: Optional[str] = None
    # The images property
    images: Optional[Images] = None
    # The modelYear property
    model_year: Optional[int] = None
    # The vin property
    vin: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> VehicleDetails_data:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: VehicleDetails_data
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return VehicleDetails_data()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .descriptions import Descriptions
        from .images import Images

        from .descriptions import Descriptions
        from .images import Images

        fields: Dict[str, Callable[[Any], None]] = {
            "batteryCapacityKWH": lambda n : setattr(self, 'battery_capacity_k_w_h', n.get_float_value()),
            "descriptions": lambda n : setattr(self, 'descriptions', n.get_object_value(Descriptions)),
            "externalColour": lambda n : setattr(self, 'external_colour', n.get_str_value()),
            "fuelType": lambda n : setattr(self, 'fuel_type', n.get_str_value()),
            "gearbox": lambda n : setattr(self, 'gearbox', n.get_str_value()),
            "images": lambda n : setattr(self, 'images', n.get_object_value(Images)),
            "modelYear": lambda n : setattr(self, 'model_year', n.get_int_value()),
            "vin": lambda n : setattr(self, 'vin', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        from .descriptions import Descriptions
        from .images import Images

        writer.write_float_value("batteryCapacityKWH", self.battery_capacity_k_w_h)
        writer.write_object_value("descriptions", self.descriptions)
        writer.write_str_value("externalColour", self.external_colour)
        writer.write_str_value("fuelType", self.fuel_type)
        writer.write_str_value("gearbox", self.gearbox)
        writer.write_object_value("images", self.images)
        writer.write_int_value("modelYear", self.model_year)
        writer.write_str_value("vin", self.vin)
        writer.write_additional_data_value(self.additional_data)
    




from enum import Enum

class CommandListItem_command(str, Enum):
    HONK_AND_FLASH = "HONK_AND_FLASH",
    HONK = "HONK",
    FLASH = "FLASH",
    LOCK = "LOCK",
    LOCK_REDUCED_GUARD = "LOCK_REDUCED_GUARD",
    UNLOCK = "UNLOCK",
    ENGINE_START = "ENGINE_START",
    ENGINE_STOP = "ENGINE_STOP",
    CLIMATIZATION_START = "CLIMATIZATION_START",
    CLIMATIZATION_STOP = "CLIMATIZATION_STOP",
    SEND_NAVI_POI = "SEND_NAVI_POI",




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from ...models.error_response import ErrorResponse
    from ...models.vehicle_details import VehicleDetails
    from .brakes.brakes_request_builder import BrakesRequestBuilder
    from .commands.commands_request_builder import CommandsRequestBuilder
    from .command_accessibility.command_accessibility_request_builder import CommandAccessibilityRequestBuilder
    from .diagnostics.diagnostics_request_builder import DiagnosticsRequestBuilder
    from .doors.doors_request_builder import DoorsRequestBuilder
    from .engine.engine_request_builder import EngineRequestBuilder
    from .engine_status.engine_status_request_builder import EngineStatusRequestBuilder
    from .fuel.fuel_request_builder import FuelRequestBuilder
    from .odometer.odometer_request_builder import OdometerRequestBuilder
    from .statistics.statistics_request_builder import StatisticsRequestBuilder
    from .tyres.tyres_request_builder import TyresRequestBuilder
    from .warnings.warnings_request_builder import WarningsRequestBuilder
    from .windows.windows_request_builder import WindowsRequestBuilder

class WithVinItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /vehicles/{vin}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, Dict[str, Any]]) -> None:
        """
        Instantiates a new WithVinItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/vehicles/{vin}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[VehicleDetails]:
        """
        Provides details about the vehicle such as model, model-year etc. Required Scope(s): conve:vehicle_relation
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[VehicleDetails]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        from ...models.error_response import ErrorResponse

        error_mapping: Dict[str, type[ParsableFactory]] = {
            "400": ErrorResponse,
            "401": ErrorResponse,
            "403": ErrorResponse,
            "404": ErrorResponse,
            "405": ErrorResponse,
            "409": ErrorResponse,
            "415": ErrorResponse,
            "422": ErrorResponse,
            "500": ErrorResponse,
            "503": ErrorResponse,
            "504": ErrorResponse,
        }
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ...models.vehicle_details import VehicleDetails

        return await self.request_adapter.send_async(request_info, VehicleDetails, error_mapping)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Provides details about the vehicle such as model, model-year etc. Required Scope(s): conve:vehicle_relation
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> WithVinItemRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: WithVinItemRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return WithVinItemRequestBuilder(self.request_adapter, raw_url)
    
    @property
    def brakes(self) -> BrakesRequestBuilder:
        """
        The brakes property
        """
        from .brakes.brakes_request_builder import BrakesRequestBuilder

        return BrakesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def command_accessibility(self) -> CommandAccessibilityRequestBuilder:
        """
        The commandAccessibility property
        """
        from .command_accessibility.command_accessibility_request_builder import CommandAccessibilityRequestBuilder

        return CommandAccessibilityRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def commands(self) -> CommandsRequestBuilder:
        """
        The commands property
        """
        from .commands.commands_request_builder import CommandsRequestBuilder

        return CommandsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def diagnostics(self) -> DiagnosticsRequestBuilder:
        """
        The diagnostics property
        """
        from .diagnostics.diagnostics_request_builder import DiagnosticsRequestBuilder

        return DiagnosticsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def doors(self) -> DoorsRequestBuilder:
        """
        The doors property
        """
        from .doors.doors_request_builder import DoorsRequestBuilder

        return DoorsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def engine(self) -> EngineRequestBuilder:
        """
        The engine property
        """
        from .engine.engine_request_builder import EngineRequestBuilder

        return EngineRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def engine_status(self) -> EngineStatusRequestBuilder:
        """
        The engineStatus property
        """
        from .engine_status.engine_status_request_builder import EngineStatusRequestBuilder

        return EngineStatusRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def fuel(self) -> FuelRequestBuilder:
        """
        The fuel property
        """
        from .fuel.fuel_request_builder import FuelRequestBuilder

        return FuelRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def odometer(self) -> OdometerRequestBuilder:
        """
        The odometer property
        """
        from .odometer.odometer_request_builder import OdometerRequestBuilder

        return OdometerRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def statistics(self) -> StatisticsRequestBuilder:
        """
        The statistics property
        """
        from .statistics.statistics_request_builder import StatisticsRequestBuilder

        return StatisticsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def tyres(self) -> TyresRequestBuilder:
        """
        The tyres property
        """
        from .tyres.tyres_request_builder import TyresRequestBuilder

        return TyresRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def warnings(self) -> WarningsRequestBuilder:
        """
        The warnings property
        """
        from .warnings.warnings_request_builder import WarningsRequestBuilder

        return WarningsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def windows(self) -> WindowsRequestBuilder:
        """
        The windows property
        """
        from .windows.windows_request_builder import WindowsRequestBuilder

        return WindowsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @dataclass
    class WithVinItemRequestBuilderGetRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .resource_instance_string import ResourceInstanceString

@dataclass
class ExteriorWarning_data(AdditionalDataHolder, Parsable):
    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: Dict[str, Any] = field(default_factory=dict)

    # The brakeLightCenterWarning property
    brake_light_center_warning: Optional[ResourceInstanceString] = None
    # The brakeLightLeftWarning property
    brake_light_left_warning: Optional[ResourceInstanceString] = None
    # The brakeLightRightWarning property
    brake_light_right_warning: Optional[ResourceInstanceString] = None
    # The daytimeRunningLightLeftWarning property
    daytime_running_light_left_warning: Optional[ResourceInstanceString] = None
    # The daytimeRunningLightRightWarning property
    daytime_running_light_right_warning: Optional[ResourceInstanceString] = None
    # The fogLightFrontWarning property
    fog_light_front_warning: Optional[ResourceInstanceString] = None
    # The fogLightRearWarning property
    fog_light_rear_warning: Optional[ResourceInstanceString] = None
    # The hazardLightsWarning property
    hazard_lights_warning: Optional[ResourceInstanceString] = None
    # The highBeamLeftWarning property
    high_beam_left_warning: Optional[ResourceInstanceString] = None
    # The highBeamRightWarning property
    high_beam_right_warning: Optional[ResourceInstanceString] = None
    # The lowBeamLeftWarning property
    low_beam_left_warning: Optional[ResourceInstanceString] = None
    # The lowBeamRightWarning property
    low_beam_right_warning: Optional[ResourceInstanceString] = None
    # The positionLightFrontLeftWarning property
    position_light_front_left_warning: Optional[ResourceInstanceString] = None
    # The positionLightFrontRightWarning property
    position_light_front_right_warning: Optional[ResourceInstanceString] = None
    # The positionLightRearLeftWarning property
    position_light_rear_left_warning: Optional[ResourceInstanceString] = None
    # The positionLightRearRightWarning property
    position_light_rear_right_warning: Optional[ResourceInstanceString] = None
    # The registrationPlateLightWarning property
    registration_plate_light_warning: Optional[ResourceInstanceString] = None
    # The reverseLightsWarning property
    reverse_lights_warning: Optional[ResourceInstanceString] = None
    # The sideMarkLightsWarning property
    side_mark_lights_warning: Optional[ResourceInstanceString] = None
    # The turnIndicationFrontLeftWarning property
    turn_indication_front_left_warning: Optional[ResourceInstanceString] = None
    # The turnIndicationFrontRightWarning property
    turn_indication_front_right_warning: Optional[ResourceInstanceString] = None
    # The turnIndicationRearLeftWarning property
    turn_indication_rear_left_warning: Optional[ResourceInstanceString] = None
    # The turnIndicationRearRightWarning property
    turn_indication_rear_right_warning: Optional[ResourceInstanceString] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ExteriorWarning_data:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ExteriorWarning_data
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ExteriorWarning_data()
    
    def get_field_deserializers(self,) -> Dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: Dict[str, Callable[[ParseNode], None]]
        """
        from .resource_instance_string import ResourceInstanceString

        from .resource_instance_string import ResourceInstanceString

        fields: Dict[str, Callable[[Any], None]] = {
            "brakeLightCenterWarning": lambda n : setattr(self, 'brake_light_center_warning', n.get_object_value(ResourceInstanceString)),
            "brakeLightLeftWarning": lambda n : setattr(self, 'brake_light_left_warning', n.get_object_value(ResourceInstanceString)),
            "brakeLightRightWarning": lambda n : setattr(self, 'brake_light_right_warning', n.get_object_value(ResourceInstanceString)),
            "daytimeRunningLightLeftWarning": lambda n : setattr(self, 'daytime_running_light_left_warning', n.get_object_value(ResourceInstanceString)),
            "daytimeRunningLightRightWarning": lambda n : setattr(self, 'daytime_running_light_right_warning', n.get_object_value(ResourceInstanceString)),
            "fogLightFrontWarning": lambda n : setattr(self, 'fog_light_front_warning', n.get_object_value(ResourceInstanceString)),
            "fogLightRearWarning": lambda n : setattr(self, 'fog_light_rear_warning', n.get_object_value(ResourceInstanceString)),
            "hazardLightsWarning": lambda n : setattr(self, 'hazard_lights_warning', n.get_object_value(ResourceInstanceString)),
            "highBeamLeftWarning": lambda n : setattr(self, 'high_beam_left_warning', n.get_object_value(ResourceInstanceString)),
            "highBeamRightWarning": lambda n : setattr(self, 'high_beam_right_warning', n.get_object_value(ResourceInstanceString)),
            "lowBeamLeftWarning": lambda n : setattr(self, 'low_beam_left_warning', n.get_object_value(ResourceInstanceString)),
            "lowBeamRightWarning": lambda n : setattr(self, 'low_beam_right_warning', n.get_object_value(ResourceInstanceString)),
            "positionLightFrontLeftWarning": lambda n : setattr(self, 'position_light_front_left_warning', n.get_object_value(ResourceInstanceString)),
            "positionLightFrontRightWarning": lambda n : setattr(self, 'position_light_front_right_warning', n.get_object_value(ResourceInstanceString)),
            "positionLightRearLeftWarning": lambda n : setattr(self, 'position_light_rear_left_warning', n.get_object_value(ResourceInstanceString)),
            "positionLightRearRightWarning": lambda n : setattr(self, 'position_light_rear_right_warning', n.get_object_value(ResourceInstanceString)),
            "registrationPlateLightWarning": lambda n : setattr(self, 'registration_plate_light_warning', n.get_object_value(ResourceInstanceString)),
            "reverseLightsWarning": lambda n : setattr(self, 'reverse_lights_warning', n.get_object_value(ResourceInstanceString)),
            "sideMarkLightsWarning": lambda n : setattr(self, 'side_mark_lights_warning', n.get_object_value(ResourceInstanceString)),
            "turnIndicationFrontLeftWarning": lambda n : setattr(self, 'turn_indication_front_left_warning', n.get_object_value(ResourceInstanceString)),
            "turnIndicationFrontRightWarning": lambda n : setattr(self, 'turn_indication_front_right_warning', n.get_object_value(ResourceInstanceString)),
            "turnIndicationRearLeftWarning": lambda n : setattr(self, 'turn_indication_rear_left_warning', n.get_object_value(ResourceInstanceString)),
            "turnIndicationRearRightWarning": lambda n : setattr(self, 'turn_indication_rear_right_warning', n.get_object_value(ResourceInstanceString)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        from .resource_instance_string import ResourceInstanceString

        writer.write_object_value("brakeLightCenterWarning", self.brake_light_center_warning)
        writer.write_object_value("brakeLightLeftWarning", self.brake_light_left_warning)
        writer.write_object_value("brakeLightRightWarning", self.brake_light_right_warning)
        writer.write_object_value("daytimeRunningLightLeftWarning", self.daytime_running_light_left_warning)
        writer.write_object_value("daytimeRunningLightRightWarning", self.daytime_running_light_right_warning)
        writer.write_object_value("fogLightFrontWarning", self.fog_light_front_warning)
        writer.write_object_value("fogLightRearWarning", self.fog_light_rear_warning)
        writer.write_object_value("hazardLightsWarning", self.hazard_lights_warning)
        writer.write_object_value("highBeamLeftWarning", self.high_beam_left_warning)
        writer.write_object_value("highBeamRightWarning", self.high_beam_right_warning)
        writer.write_object_value("lowBeamLeftWarning", self.low_beam_left_warning)
        writer.write_object_value("lowBeamRightWarning", self.low_beam_right_warning)
        writer.write_object_value("positionLightFrontLeftWarning", self.position_light_front_left_warning)
        writer.write_object_value("positionLightFrontRightWarning", self.position_light_front_right_warning)
        writer.write_object_value("positionLightRearLeftWarning", self.position_light_rear_left_warning)
        writer.write_object_value("positionLightRearRightWarning", self.position_light_rear_right_warning)
        writer.write_object_value("registrationPlateLightWarning", self.registration_plate_light_warning)
        writer.write_object_value("reverseLightsWarning", self.reverse_lights_warning)
        writer.write_object_value("sideMarkLightsWarning", self.side_mark_lights_warning)
        writer.write_object_value("turnIndicationFrontLeftWarning", self.turn_indication_front_left_warning)
        writer.write_object_value("turnIndicationFrontRightWarning", self.turn_indication_front_right_warning)
        writer.write_object_value("turnIndicationRearLeftWarning", self.turn_indication_rear_left_warning)
        writer.write_object_value("turnIndicationRearRightWarning", self.turn_indication_rear_right_warning)
        writer.write_additional_data_value(self.additional_data)
    

